<?php

// phpcs:disable WordPress.Security.NonceVerification.Recommended

declare( strict_types=1 );

namespace WPDesk\ShopMagicCart\Admin\CartsMenu;

use ShopMagicVendor\WPDesk\View\Resolver\DirResolver;
use WPDesk\ShopMagic\Admin\SelectAjaxField\CustomerSelectAjax;
use ShopMagicVendor\WPDesk\Forms\Field\SubmitField;
use ShopMagicVendor\WPDesk\Forms\Field\SelectField;
use ShopMagicVendor\WPDesk\Forms\Form;
use ShopMagicVendor\WPDesk\View\Renderer\SimplePhpRenderer;
use WPDesk\ShopMagic\Admin\TableList\AbstractTableList;
use WPDesk\ShopMagic\Helper\WordPressFormatHelper;
use WPDesk\ShopMagicCart\Cart\Cart;
use WPDesk\ShopMagicCart\Cart\CartFactory;

/**
 * WordPress WP_List_Table for carts list.
 */
final class TableList extends AbstractTableList {

	/** @var Form\FormWithFields */
	protected $form_filter;

	/** @var string */
	protected $singular_name = 'cart';

	/** @var string */
	protected $plural_name = 'carts';

	/** @var bool */
	protected $use_ajax = false;

	protected $order_key = 'last_modified';

	/** @var string */
	protected $item_per_page_option = 'cart_items_per_page';

	protected function get_fields(): array {
		return [
			( new SelectField() )
				->set_options(
					[
						''              => __( 'Cart status', 'shopmagic-abandoned-carts' ),
						'ordered'       => __( 'Ordered', 'shopmagic-abandoned-carts' ),
						'abandoned'     => __( 'Abandoned', 'shopmagic-abandoned-carts' ),
						'recovered'     => __( 'Recovered', 'shopmagic-abandoned-carts' ),
					]
				)
				->set_name( 'status' ),
			( new CustomerSelectAjax() )
				->set_name( 'user_id' ),
			( new SubmitField() )
				->set_name( 'submit' )
				->add_class( 'button' )
				->set_label( __( 'Filter', 'shopmagic-abandoned-carts' ) ),
		];
	}

	/** @return string[] */
	public function get_columns(): array {
		return [
			'cb'         => '<input type="checkbox" />',
			'id'         => __( 'ID', 'shopmagic-abandoned-carts' ),
			'customer'   => __( 'Customer', 'shopmagic-abandoned-carts' ),
			'email'      => esc_html__( 'Email', 'shopmagic-abandoned-carts' ),
			'item_count' => __( 'Items', 'shopmagic-abandoned-carts' ),
			'total'      => __( 'Cart Total', 'shopmagic-abandoned-carts' ),
			'updated'    => __( 'Time', 'shopmagic-abandoned-carts' ),
			'status'     => __( 'Status', 'shopmagic-abandoned-carts' ),
		];
	}

	/** @return string[] */
	protected function get_bulk_actions(): array {
		return [
			'bulk_delete' => __( 'Delete', 'shopmagic-abandoned-carts' ),
		];
	}

	/** @return void */
	protected function process_bulk_delete() {
		$post_type_object = get_post_type_object( $this->screen->post_type );
		if ( $post_type_object && current_user_can( $post_type_object->cap->delete_posts ) ) {
			/** @var int[] $delete_ids */
			$delete_ids = isset( $_GET['bulk-delete'] ) ? esc_sql( wp_unslash( $_GET['bulk-delete'] ) ) : []; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			foreach ( $delete_ids as $id ) {
				$this->table->delete_by_where( [ 'id' => $id ] );
			}
		}
	}

	protected function column_email( Cart $cart ): string {
		return $cart->get_customer()->get_email();
	}

	/** @return array<string, array{string, bool}> */
	protected function get_sortable_columns(): array {
		return [
			'id'       => [ 'id', false ],
			'customer' => [ 'customer', false ],
			'email'    => [ 'email', false ],
			'updated'  => [ 'last_modified', false ],
			'total'    => [ 'total', false ],
			'status'   => [ 'status', false ],
		];
	}

	/** @param Cart $item */
	protected function column_cb( $item ): string {
		return sprintf(
			'<input type="checkbox" name="bulk-delete[]" value="%s" />',
			$item->get_id()
		);
	}

	protected function column_id( Cart $cart ): string {
		return sprintf( '#%s', esc_html( (string) $cart->get_id() ) );
	}

	protected function column_status( Cart $cart ): string {
		switch ( $cart->get_status() ) {
			case CartFactory::STATUS_ORDERED:
				$status_name        = esc_html__( 'Ordered', 'shopmagic-abandoned-carts' );
				$status_description = esc_html__( 'Ordered immediately', 'shopmagic-abandoned-carts' );
				$status_class       = '';
				break;
			case CartFactory::STATUS_ABANDONED:
				$status_name        = esc_html__( 'Abandoned', 'shopmagic-abandoned-carts' );
				$status_description = esc_html__( 'Cart is abandoned', 'shopmagic-abandoned-carts' );
				$status_class       = 'unknown';
				break;
			case CartFactory::STATUS_ACTIVE:
				$status_name        = esc_html__( 'Active', 'shopmagic-abandoned-carts' );
				$status_description = esc_html__( 'Cart currently active', 'shopmagic-abandoned-carts' );
				$status_class       = 'queued';
				break;
			case CartFactory::STATUS_RECOVERED:
				$status_name        = esc_html__( 'Recovered', 'shopmagic-abandoned-carts' );
				$status_description = esc_html__( 'Cart recovered', 'shopmagic-abandoned-carts' );
				$status_class       = 'completed';
				break;
			default:
				$status_name        = esc_html__( 'Unknown', 'shopmagic-abandoned-carts' );
				$status_description = esc_html__( 'Unknown', 'shopmagic-abandoned-carts' );
				$status_class       = '';
		}

		return sprintf( '<mark class="outcome-status status-%s tips" data-tip="%s"><span>%s</span></mark>', $status_class, $status_description, $status_name );
	}

	protected function column_customer( Cart $cart ): string {
		return \WPDesk\ShopMagic\Admin\Outcome\TableList::render_customer_column( $cart->get_customer() );
	}

	protected function column_updated( Cart $cart ): string {
		return $this->render_timestamp( $cart->get_last_modified() );
	}

	protected function column_total( Cart $cart ): string {
		return wc_price(
			$cart->get_total(),
			[
				'currency' => $cart->get_currency(),
			]
		);
	}

	protected function column_item_count( Cart $cart ): string {
		$renderer = new SimplePhpRenderer( ( new DirResolver( __DIR__ . '/templates' ) ) );
		return $renderer->render( 'items', [ 'items' => $cart->get_items() ] );
	}

	private function render_timestamp( \DateTimeInterface $timestamp ): string {
		$timestamp_format = 'M d, Y H:i';
		$timestamp_format = apply_filters( 'shopmagic/carts/carts/timestamp_format', $timestamp_format );

		return WordPressFormatHelper::format_wp_datetime( $timestamp, $timestamp_format );
	}

	protected function column_options( Cart $cart ): string {
		return sprintf( '<a href="%s">View</a>', SingleCart::get_url( $cart ) );
	}

}
