<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagicCart\Cart;

use DateTimeImmutable;
use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Customer\CustomerFactory;
use WPDesk\ShopMagic\Database\Abstraction\DAO;
use WPDesk\ShopMagic\Helper\WordPressFormatHelper;

final class Cart implements DAO\Item {

	/** @var int|null */
	private $id;

	/** @var string */
	private $status;

	/** @var Customer */
	private $customer;

	/** @var \DateTimeInterface */
	private $last_modified;

	/** @var \DateTimeInterface */
	private $created;

	/** @var CartProductItem[] */
	private $items;

	/** @var array */
	private $coupons;

	/** @var array */
	private $fees;

	/** @var float */
	private $shipping_tax_total;

	/** @var float */
	private $shipping_total;

	/** @var float */
	private $total;

	/** @var string */
	private $token;

	/** @var string */
	private $currency;

	/** @var bool */
	private $changed = false;

	/** @var float */
	private $calculated_tax_total = 0;

	/** @var float */
	private $calculated_subtotal = 0;

	public function __construct( int $id, string $status, Customer $customer, \DateTimeInterface $last_modified, \DateTimeInterface $created, array $items, array $coupons, array $fees, float $shipping_tax_total, float $shipping_total, float $total, string $token, string $currency ) {
		$this->id                 = $id;
		$this->status             = $status;
		$this->customer           = $customer;
		$this->last_modified      = $last_modified;
		$this->created            = $created;
		$this->items              = $this->map_array_items( $items );
		$this->coupons            = $coupons;
		$this->fees               = $fees;
		$this->shipping_tax_total = $shipping_tax_total;
		$this->shipping_total     = $shipping_total;
		$this->total              = $total;
		$this->token              = $token;
		$this->currency           = $currency;
	}

	/**
	 * @param array $items
	 *
	 * @return CartProductItem[]
	 */
	private function map_array_items( array $items ): array {
		return array_map(
			function ( $item_data ) {
				return new CartProductItem( $item_data );
			},
			$items
		);
	}

	public function has_changed(): bool {
		return empty( $this->id ) || $this->changed;
	}

	public function get_changed_fields(): array {
		return [];
	}

	/** @return void */
	public function sync_order( \WC_Abstract_Order $order ) {
		$order->add_meta_data( 'shopmagic_cart_id', $this->id );
		$this->changed = true;
		if ( $this->status === CartFactory::STATUS_ABANDONED ) {
			$this->status = CartFactory::STATUS_RECOVERED;
		} else {
			$this->status = CartFactory::STATUS_ORDERED;
		}
	}

	private function needs_update( string $token ): bool {
		$created = $this->created->format( WordPressFormatHelper::MYSQL_DATETIME_FORMAT );
		$updated = $this->last_modified->format( WordPressFormatHelper::MYSQL_DATETIME_FORMAT );

		return ( $created !== $updated ) ||
			   ( $this->token !== $token );
	}

	/**
	 * Updates the stored cart with the current time and cart items
	 *
	 * @return void
	 */
	public function sync( \WC_Cart $wc_cart, Customer $customer, string $token ) {
		$this->last_modified = new \DateTimeImmutable();

		if ( $this->needs_update( $token ) ) {
			$this->changed = true;
		}

		$this->items = $this->map_array_items( WC()->cart->get_cart_for_session() );

		$coupon_data = [];
		foreach ( $wc_cart->get_applied_coupons() as $coupon_code ) {
			$coupon_data[ $coupon_code ] = [
				'discount_incl_tax' => $wc_cart->get_coupon_discount_amount( $coupon_code, false ),
				'discount_excl_tax' => $wc_cart->get_coupon_discount_amount( $coupon_code ),
				'discount_tax'      => $wc_cart->get_coupon_discount_tax_amount( $coupon_code ),
			];
		}

		$this->coupons            = $coupon_data;
		$this->token              = $token;
		$this->fees               = $wc_cart->get_fees();
		$this->currency           = get_woocommerce_currency();
		$this->customer           = $customer;
		$this->shipping_tax_total = $wc_cart->shipping_tax_total;
		$this->shipping_total     = $wc_cart->shipping_total;

		$this->calculate_totals();

		if ( $this->status === CartFactory::STATUS_NEW ) {
			$this->status = CartFactory::STATUS_ACTIVE;
		}
	}

	/** @return void */
	private function calculate_totals() {
		$this->calculated_subtotal  = 0;
		$this->calculated_tax_total = 0;
		$this->total                = 0;

		$tax_display = get_option( 'woocommerce_tax_display_cart' );

		foreach ( $this->get_items() as $item ) {
			$this->calculated_tax_total += $item->get_line_subtotal_tax();
			$this->total                += $item->get_line_subtotal() + $item->get_line_subtotal_tax();
			$this->calculated_subtotal  += $tax_display === 'excl' ? $item->get_line_subtotal() : $item->get_line_subtotal() + $item->get_line_subtotal_tax();
		}

		foreach ( $this->get_coupons() as $coupon ) {
			$this->total                -= $coupon['discount_incl_tax'];
			$this->calculated_tax_total -= $coupon['discount_tax'];
		}

		foreach ( $this->get_fees() as $fee ) {
			$this->total                += ( $fee->amount + $fee->tax );
			$this->calculated_tax_total += $fee->tax;
		}

		$this->calculated_tax_total += $this->get_shipping_tax_total();
		$this->total                += $this->get_shipping_total();
		$this->total                += $this->get_shipping_tax_total();
	}


	/** @return int|null */
	public function get_id() {
		return $this->id;
	}

	public function get_status_translated(): string {
		switch ( $this->status ) {
			case CartFactory::STATUS_ACTIVE:
				return __( 'Active', 'shopmagic-abandoned-carts' );
			case CartFactory::STATUS_ABANDONED:
				return __( 'Abandoned', 'shopmagic-abandoned-carts' );
			case CartFactory::STATUS_ORDERED:
				return __( 'Ordered', 'shopmagic-abandoned-carts' );
			case CartFactory::STATUS_RECOVERED:
				return __( 'Recovered', 'shopmagic-abandoned-carts' );
		}

		return '';
	}

	public function get_customer(): Customer {
		return $this->customer;
	}

	public function get_last_modified(): \DateTimeInterface {
		return $this->last_modified;
	}

	public function get_created(): \DateTimeInterface {
		return $this->created;
	}

	/** @return CartProductItem[] */
	public function get_items(): array {
		return $this->items;
	}

	public function get_products_quantity_count(): float {
		return array_reduce(
			$this->get_items(),
			static function ( $carry, CartProductItem $item ) {
				return $carry + $item->get_quantity();
			},
			0
		);
	}

	public function get_coupons(): array {
		return $this->coupons;
	}

	private function get_fees(): array {
		return $this->fees;
	}

	public function get_shipping_tax_total(): float {
		return (float) $this->shipping_tax_total;
	}

	public function get_shipping_total(): float {
		return (float) $this->shipping_total;
	}

	public function get_total(): float {
		return (float) $this->total;
	}

	public function get_token(): string {
		return $this->token;
	}

	public function get_currency(): string {
		return $this->currency;
	}

	public function normalize(): array {
		$vars = get_object_vars( $this );

		$vars['last_modified'] = WordPressFormatHelper::datetime_as_mysql( $this->last_modified );
		$vars['created']       = WordPressFormatHelper::datetime_as_mysql( $this->created );
		$vars['items']         = json_encode( $this->items );
		$vars['coupons']       = json_encode( $this->coupons );
		$vars['fees']          = json_encode( $this->fees );

		if ( $this->customer->is_guest() ) {
			$vars['guest_id'] = CustomerFactory::convert_customer_guest_id_to_number( (string) $this->customer->get_id() );
			$vars['user_id']  = null;
		} else {
			$vars['user_id']  = $this->customer->get_id();
			$vars['guest_id'] = null;
		}

		return $vars;
	}

	public function set_last_inserted_id( int $id ) {
		$this->id = $id;
	}

	/** @return void */
	public function set_status_abandoned() {
		if ( $this->status !== CartFactory::STATUS_ABANDONED ) {
			$this->status  = CartFactory::STATUS_ABANDONED;
			$this->changed = true;
		}
	}

	public function is_status_abandoned(): bool {
		return $this->status === CartFactory::STATUS_ABANDONED;
	}

	/** @return void */
	public function append_to_wc_cart( \WC_Cart $wc_cart ) {
		$notices_backup = wc_get_notices();
		try {
			foreach ( $this->get_items() as $item ) {
				$item->append_to_wc_cart( $wc_cart );
			}

			foreach ( $this->get_coupons() as $coupon_code => $coupon_data ) {
				if ( ! $wc_cart->has_discount( $coupon_code ) ) {
					$wc_cart->add_discount( $coupon_code );
				}
			}
		} finally {
			WC()->session->set( 'wc_notices', $notices_backup );
		}
	}

	public function get_status(): string {
		return $this->status;
	}
}
