<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagicCart\Cart;

use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Customer\CustomerFactory;
use WPDesk\ShopMagic\Database\Abstraction\DAO;
use WPDesk\ShopMagic\Database\Abstraction\AbstractSingleTable;
use WPDesk\ShopMagic\Exception\CannotProvideItemException;
use WPDesk\ShopMagicCart\Database\UpdateCartTable;

final class CartTable extends AbstractSingleTable {

	/** @var DAO\ItemFactory */
	private $cart_factory;

	public function __construct( DAO\ItemFactory $cart_factory ) {
		parent::__construct();
		$this->cart_factory = $cart_factory;
	}

	public function get_factory(): DAO\ItemFactory {
		return $this->cart_factory;
	}

	protected function get_columns(): array {
		return [
			'id',
			'status',
			'user_id',
			'guest_id',
			'last_modified',
			'created',
			'items',
			'coupons',
			'fees',
			'shipping_tax_total',
			'shipping_total',
			'total',
			'token',
			'currency',
		];
	}

	protected function get_name(): string {
		return UpdateCartTable::get_name();
	}

	protected function get_primary_key(): array {
		return [ 'id' ];
	}

	public function is_valid_item( DAO\Item $item ): bool {
		return $item instanceof Cart;
	}

	/**
	 * @param string $token
	 *
	 * @return Cart
	 *
	 * @throws CannotProvideItemException
	 */
	public function get_by_token( string $token ): Cart {
		/** @var Cart $cart */
		$cart = $this->get_single_by_where( [ 'token' => $token ] );

		return $cart;
	}

	/**
	 * @param Customer $customer
	 *
	 * @return Cart
	 *
	 * @throws CannotProvideItemException
	 */
	public function get_by_customer( Customer $customer ): Cart {
		$status_where = [
			[
				'field'     => 'status',
				'condition' => '<>',
				'value'     => CartFactory::STATUS_ORDERED,
			],
			[
				'field'     => 'status',
				'condition' => '<>',
				'value'     => CartFactory::STATUS_RECOVERED,
			],
		];
		if ( $customer->is_guest() ) {
			$cart = $this->get_single_by_where(
				array_merge(
					$status_where,
					[
						'guest_id' => CustomerFactory::convert_customer_guest_id_to_number( $customer->get_id() ),
					]
				)
			);
		} else {
			$cart = $this->get_single_by_where(
				array_merge(
					$status_where,
					[
						'user_id' => $customer->get_id(),
					]
				)
			);
		}

		/** @var Cart $cart */
		return $cart;
	}

	public function save( DAO\Item $item ): bool {
		do_action( 'shopmagic/carts/before_cart_save', $item );
		$result = parent::save( apply_filters( 'shopmagic/carts/cart_to_save', $item ) );
		do_action( 'shopmagic/carts/after_cart_save', $item );

		return $result;
	}
}
