<?php

namespace WPDesk\ShopMagicCart\Event;

use Psr\Container\NotFoundExceptionInterface;
use ShopMagicVendor\WPDesk\Forms\Field\InputNumberField;
use WPDesk\ShopMagic\Admin\Settings\Settings as CoreSettings;
use WPDesk\ShopMagic\AutomationOutcome\OutcomeInformationRepository;
use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Customer\CustomerFactory;
use WPDesk\ShopMagic\Event\BasicEvent;
use WPDesk\ShopMagic\Event\DeferredStateCheck\SupportsDeferredCheck;
use WPDesk\ShopMagic\Event\EventFactory;
use WPDesk\ShopMagicCart\Admin\Settings;
use WPDesk\ShopMagicCart\Cart\Cart;
use WPDesk\ShopMagicCart\Cart\CartTable;

final class AbandonedCartEvent extends BasicEvent implements SupportsDeferredCheck {
	const FIELD_NAME_PAUSE_PERIOD = 'pause_period';

	/** @var CustomerFactory */
	private $customer_factory;

	/** @var CartTable */
	private $cart_table;

	/** @var OutcomeInformationRepository */
	private $outcome_information;

	/**@var Cart */
	private $cart;

	/** @var Customer */
	private $customer;

	public function __construct( CustomerFactory $customer_factory, CartTable $cart_table, OutcomeInformationRepository $outcome_information ) {
		$this->customer_factory    = $customer_factory;
		$this->cart_table          = $cart_table;
		$this->outcome_information = $outcome_information;
	}

	public function get_fields(): array {
		return [
			( new InputNumberField() )
				->set_name( self::FIELD_NAME_PAUSE_PERIOD )
				->set_attribute( 'min', 1 )
				->set_label( __( 'Pause period for customer (days)', 'shopmagic-abandoned-carts' ) )
				->set_description( __( 'Can be used to ensure that this event will trigger only once in a specified time period.', 'shopmagic-abandoned-carts' ) ),
		];
	}

	public function get_name(): string {
		return __( 'Abandoned Cart', 'shopmagic-abandoned-carts' );
	}

	public function get_description(): string {
		$abandoned_timeout = Settings::get_option( 'abandoned_cart_timeout' );

		return sprintf(
			// translators: %d amount of minutes to consider abandoned.
			_n(
				'Triggered %d minute after the cart is considered abandoned.',
				'Triggered %d minutes after the cart is considered abandoned.',
				$abandoned_timeout,
				'shopmagic-abandoned-carts'
			),
			$abandoned_timeout
		) . ' <a href="' . CoreSettings::get_url( Settings::get_tab_slug() ) . '" target="_blank">' . __( 'Change this time', 'shopmagic-abandoned-carts' ) . '</a>';
	}

	public function get_group_slug(): string {
		return EventFactory::GROUP_CARTS;
	}

	public function jsonSerialize(): array {
		return array_merge(
			parent::jsonSerialize(),
			[
				'cart_id'     => $this->cart->get_id(),
				'customer_id' => $this->customer->get_id(),
			]
		);
	}

	/** @return void */
	public function set_from_json( array $serializedJson ) {
		parent::set_from_json( $serializedJson );
		$this->cart     = $this->cart_table->get_by_primary( (string) $serializedJson['cart_id'] );
		$this->customer = $this->customer_factory->create_from_id( $serializedJson['customer_id'] );
	}

	public function get_provided_data_domains(): array {
		return array_merge(
			parent::get_provided_data_domains(),
			[ Cart::class, Customer::class ]
		);
	}

	public function get_provided_data(): array {
		$data[ Cart::class ]     = $this->cart;
		$data[ Customer::class ] = $this->customer;

		return array_merge(
			parent::get_provided_data(),
			$data
		);
	}

	/** @return void */
	public function process_event( Cart $cart ) {
		$this->cart     = $cart;
		$this->customer = $cart->get_customer();

		try {
			if ( $this->fields_data->has( self::FIELD_NAME_PAUSE_PERIOD ) ) {
				$pause_period = $this->fields_data->get( self::FIELD_NAME_PAUSE_PERIOD );
				if ( ! empty( $pause_period ) && $this->outcome_information->get_done_automation_count_with_time( $this->automation->get_id(), $this->customer->get_id(), (int) $pause_period ) > 0 ) {
					return;
				}
			}
		} catch ( NotFoundExceptionInterface $e ) {
			// fallback for older version without the field.
		}
		$this->trigger_automation();
	}

	/** @return void */
	public function initialize() {
		add_action( 'shopmagic/carts/abandoned_cart', [ $this, 'process_event' ] );
	}

	public function is_event_still_valid(): bool {
		return $this->cart->is_status_abandoned();
	}
}
