<?php

namespace WPDesk\ShopMagicCart\HookEmitter;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Psr\Log\LoggerInterface;
use ShopMagicCartVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\ShopMagic\Database\Abstraction\DAO\Collection;
use WPDesk\ShopMagic\Helper\WordPressFormatHelper;
use WPDesk\ShopMagicCart\Admin\Settings;
use WPDesk\ShopMagicCart\Cart\Cart;
use WPDesk\ShopMagicCart\Cart\CartFactory;
use WPDesk\ShopMagicCart\Cart\CartTable;

/**
 * Abandon carts that are overdue.
 */
final class AbandonedCart implements Hookable, LoggerAwareInterface {
	use LoggerAwareTrait;

	/** @var CartTable */
	private $cart_table;

	/** @var int */
	private $abandon_when;

	public function __construct( CartTable $cart_table, LoggerInterface $logger ) {
		$this->cart_table   = $cart_table;
		$this->abandon_when = max( (int) Settings::get_option( Settings::TIMEOUT, Settings::TIMEOUT_DEFAULT ), Settings::TIMEOUT_DEFAULT );
		$this->logger       = $logger;
	}

	/** @return void */
	public function hooks() {
		add_action( 'shopmagic/core/cron/one_minute', [ $this, 'abandon_carts' ] );
	}

	/**
	 * @return Collection<Cart>
	 */
	private function find_carts_to_abandon(): Collection {
		$cut_time = ( new \DateTimeImmutable( 'now', wp_timezone() ) )
			->modify( "-{$this->abandon_when} minutes" );

		return $this->cart_table->get_all(
			[
				[
					'field'     => 'last_modified',
					'condition' => '<=',
					'value'     => WordPressFormatHelper::datetime_as_mysql( $cut_time ),
				],
				[
					'field'     => 'status',
					'condition' => '=',
					'value'     => CartFactory::STATUS_ACTIVE,
				],
			]
		);
	}

	/**
	 * @return void
	 * @internal
	 */
	public function abandon_carts() {
		/** @var Cart $cart */
		foreach ( $this->find_carts_to_abandon() as $cart ) {
			$cart->set_status_abandoned();
			if ( $this->cart_table->save( $cart ) ) {
				$this->logger->debug( sprintf( 'Abandoning cart %d', $cart->get_id() ) );
				do_action( 'shopmagic/carts/abandoned_cart', $cart );
			}
		}
	}
}
