<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagicCart;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Psr\Log\NullLogger;
use ShopMagicVendor\WPDesk\Forms\Field\CheckboxField;
use WPDesk\ShopMagic\Integration\ExternalPluginsAccess;
use ShopMagicCartVendor\WPDesk\Notice\Notice;
use ShopMagicCartVendor\WPDesk_Plugin_Info;
use ShopMagicCartVendor\WPDesk\PluginBuilder\Plugin\AbstractPlugin;
use ShopMagicCartVendor\WPDesk\PluginBuilder\Plugin\HookableCollection;
use ShopMagicCartVendor\WPDesk\PluginBuilder\Plugin\HookableParent;
use WPDesk\ShopMagic\Placeholder\TemplateRendererForPlaceholders;

/**
 * Main plugin class. The most important flow decisions are made here.
 *
 * @package WPDesk\ShopMagicForWP
 */
final class Plugin extends AbstractPlugin implements LoggerAwareInterface, HookableCollection {
	use LoggerAwareTrait;
	use HookableParent;

	public function __construct( WPDesk_Plugin_Info $plugin_info ) {
		parent::__construct( $plugin_info ); // @phpstan-ignore-line
		$this->setLogger( new NullLogger() );

		$this->plugin_url       = $this->plugin_info->get_plugin_url();
		$this->plugin_namespace = $this->plugin_info->get_text_domain();
		$this->docs_url         = 'https://docs.shopmagic.app/?utm_source=user-site&utm_medium=quick-link&utm_campaign=docs';
		$this->support_url      = 'https://shopmagic.app/support/?utm_source=user-site&utm_medium=quick-link&utm_campaign=support';
	}

	/**
	 * Integrate with WordPress and with other plugins using action/filter system.
	 *
	 * @return void
	 */
	public function hooks() {
		parent::hooks();

		add_action(
			'shopmagic/core/initialized/v2',
			function ( ExternalPluginsAccess $core ) {
				$shopmagic_version = $core->get_version();
				if ( version_compare( $shopmagic_version, '3', '>=' ) ) {
					new Notice(
						sprintf(
							// translators: %s ShopMagic version.
							__(
								'This version of ShopMagic Abandoned Carts plugin is not compatible with ShopMagic %s. Please upgrade ShopMagic Abandoned Carts to the newest version.',
								'shopmagic-abandoned-carts'
							),
							$shopmagic_version
						)
					);

					return;
				}

				$this->logger = $core->get_logger();

				global $wpdb;
				$db_schema = new Database\DatabaseSchema( $this->plugin_info->get_plugin_file_name(), $wpdb );
				if ( $db_schema->is_old_database() ) {
					$db_schema->install();
				}

				$cart_dao = new Cart\CartTable( new Cart\CartFactory( $core->get_customer_factory() ) );
				if ( is_admin() ) {
					( new Admin\Admin( $this->plugin_info->get_plugin_url(), $this->plugin_info->get_version() ) )->hooks();
					( new Admin\CartsMenu\ListMenu( $cart_dao, new Cart\CartStatistics( $wpdb ) ) )->hooks();
					( new Admin\CartsMenu\SingleCart( $cart_dao ) )->hooks();
					( new TestData\CartTestData( $cart_dao ) )->hooks();

					add_filter(
						'shopmagic/core/settings/tabs',
						static function ( $settings_tab ) {
							$settings_tab[ Admin\Settings::get_tab_slug() ] = new Admin\Settings();

							return $settings_tab;
						}
					);
				}

				add_filter(
					'shopmagic/core/events',
					static function ( $events ) use ( $cart_dao, $core ) {
						$new_events = [
							'cart_abandoned_event' => new Event\AbandonedCartEvent( $core->get_customer_factory(), $cart_dao, $core->get_outcome_information() ),
						];

							return array_merge( $events, $new_events );
					}
				);

				add_filter(
					'shopmagic/core/filters',
					static function ( $filters ) {
						$new_filters = [
							'cart_date_created'    => new Filter\CartDateCreated(),
							'cart_item_categories' => new Filter\CartItemCategories(),
							'cart_item_count'      => new Filter\CartItemCount(),
							'cart_items'           => new Filter\CartItems(),
							'cart_total'           => new Filter\CartTotal(),
						];

						return array_merge( $filters, $new_filters );
					}
				);

				add_filter(
					'shopmagic/core/placeholders',
					static function ( $placeholders ) {
						$new_placeholders = [
							new Placeholder\CartItemCount(),
							new Placeholder\CartItems( TemplateRendererForPlaceholders::get_placeholder_template_renderer( 'products_ordered' ) ),
							new Placeholder\CartTotal(),
							new Placeholder\CartLink(),
						];

						return array_merge( $placeholders, $new_placeholders );
					}
				);

				( new Interceptor\CartInterceptor( $cart_dao, $core->get_customer_provider(), $this->logger ) )->hooks();
				( new HookEmitter\AbandonedCart( $cart_dao, $this->logger ) )->hooks();
				if ( \WPDesk\ShopMagicCart\Admin\Settings::get_option( \WPDesk\ShopMagicCart\Admin\Settings::EXPIRATION ) === CheckboxField::VALUE_TRUE ) {
					( new HookEmitter\CartExpiration( $cart_dao, $this->logger ) )->hooks();
				}
				( new Frontend\CartRestore( $cart_dao, $this->logger ) )->hooks();
				if ( \WPDesk\ShopMagicCart\Admin\Settings::get_option( \WPDesk\ShopMagicCart\Admin\Settings::ENABLE_EXIT_POPUP ) === CheckboxField::VALUE_TRUE ) {
					( new Frontend\ExitIntent( $core->get_customer_provider(), $this->plugin_info->get_plugin_url() . '/assets', $this->plugin_info->get_version() ) )->hooks();
				}
			},
			10,
			3
		);
	}
}
